/*
 * $Id$
 */

package org.apache.maven.model.io.jdom;

  //---------------------------------/
 //- Imported classes and packages -/
//---------------------------------/

import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Properties;
import org.apache.maven.model.Activation;
import org.apache.maven.model.ActivationFile;
import org.apache.maven.model.ActivationOS;
import org.apache.maven.model.ActivationProperty;
import org.apache.maven.model.Build;
import org.apache.maven.model.BuildBase;
import org.apache.maven.model.CiManagement;
import org.apache.maven.model.ConfigurationContainer;
import org.apache.maven.model.Contributor;
import org.apache.maven.model.Dependency;
import org.apache.maven.model.DependencyManagement;
import org.apache.maven.model.DeploymentRepository;
import org.apache.maven.model.Developer;
import org.apache.maven.model.DistributionManagement;
import org.apache.maven.model.Exclusion;
import org.apache.maven.model.Extension;
import org.apache.maven.model.FileSet;
import org.apache.maven.model.IssueManagement;
import org.apache.maven.model.License;
import org.apache.maven.model.MailingList;
import org.apache.maven.model.Model;
import org.apache.maven.model.ModelBase;
import org.apache.maven.model.Notifier;
import org.apache.maven.model.Organization;
import org.apache.maven.model.Parent;
import org.apache.maven.model.PatternSet;
import org.apache.maven.model.Plugin;
import org.apache.maven.model.PluginConfiguration;
import org.apache.maven.model.PluginContainer;
import org.apache.maven.model.PluginExecution;
import org.apache.maven.model.PluginManagement;
import org.apache.maven.model.Prerequisites;
import org.apache.maven.model.Profile;
import org.apache.maven.model.Relocation;
import org.apache.maven.model.ReportPlugin;
import org.apache.maven.model.ReportSet;
import org.apache.maven.model.Reporting;
import org.apache.maven.model.Repository;
import org.apache.maven.model.RepositoryBase;
import org.apache.maven.model.RepositoryPolicy;
import org.apache.maven.model.Resource;
import org.apache.maven.model.Scm;
import org.apache.maven.model.Site;
import org.codehaus.plexus.util.xml.Xpp3Dom;
import org.jdom.Content;
import org.jdom.DefaultJDOMFactory;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Text;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * Class MavenJDOMWriter.
 * 
 * @version $Revision$ $Date$
 */
public class MavenJDOMWriter {


      //--------------------------/
     //- Class/Member Variables -/
    //--------------------------/

    /**
     * Field factory
     */
    private DefaultJDOMFactory factory;

    /**
     * Field lineSeparator
     */
    private String lineSeparator;


      //----------------/
     //- Constructors -/
    //----------------/

    public MavenJDOMWriter() {
        factory = new DefaultJDOMFactory();
        lineSeparator = System.getProperty("line.separator");
    } //-- org.apache.maven.model.io.jdom.MavenJDOMWriter()


      //-----------/
     //- Methods -/
    //-----------/

    /**
     * Method findAndReplaceProperties
     * 
     * @param counter
     * @param props
     * @param name
     * @param parent
     */
    public Element findAndReplaceProperties(Counter counter, Element parent, String name, Map props)
    {
        boolean shouldExist = props != null && ! props.isEmpty();
        Element element = updateElement(counter, parent, name, shouldExist);
        if (shouldExist) {
            Iterator it = props.keySet().iterator();
            Counter innerCounter = new Counter();
            while (it.hasNext()) {
                String key = (String) it.next();
                findAndReplaceSimpleElement(innerCounter, element, key, (String)props.get(key));
                }
            ArrayList lst = new ArrayList(props.keySet());
            it = element.getChildren().iterator();
            while (it.hasNext()) {
                Element elem = (Element) it.next();
                String key = elem.getName();
                if (!lst.contains(key)) {
                    it.remove();
                }
            }
        }
        return element;
    } //-- Element findAndReplaceProperties(Counter, Element, String, Map) 

    /**
     * Method findAndReplaceSimpleElement
     * 
     * @param counter
     * @param text
     * @param name
     * @param parent
     */
    public Element findAndReplaceSimpleElement(Counter counter, Element parent, String name, String text)
    {
        boolean shouldExist = text != null && text.trim().length() > 0;
        Element element = updateElement(counter, parent, name, shouldExist);
        if (shouldExist) {
            element.setText(text);
        }
        return element;
    } //-- Element findAndReplaceSimpleElement(Counter, Element, String, String) 

    /**
     * Method findAndReplaceSimpleLists
     * 
     * @param counter
     * @param childName
     * @param parentName
     * @param list
     * @param parent
     */
    public Element findAndReplaceSimpleLists(Counter counter, Element parent, java.util.Collection list, String parentName, String childName)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, parentName, shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren(childName, element.getNamespace()).iterator();
            if (! elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                String value = (String) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element(childName, element.getNamespace());
                    element.addContent(el);
                }
                el.setText(value);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
        return element;
    } //-- Element findAndReplaceSimpleLists(Counter, Element, java.util.Collection, String, String) 

    /**
     * Method findAndReplaceXpp3DOM
     * 
     * @param counter
     * @param dom
     * @param name
     * @param parent
     */
    public Element findAndReplaceXpp3DOM(Counter counter, Element parent, String name, Xpp3Dom dom)
    {
        boolean shouldExist = dom != null && (dom.getChildCount() > 0 || dom.getValue() != null);
        Element element = updateElement(counter, parent, name, shouldExist);
        if (shouldExist) {
            replaceXpp3DOM(element, dom);
        }
        return element;
    } //-- Element findAndReplaceXpp3DOM(Counter, Element, String, Xpp3Dom) 

    /**
     * Method insertAtPreferredLocation
     * 
     * @param parent
     * @param counter
     * @param child
     */
    public void insertAtPreferredLocation(Element parent, Element child, Counter counter)
    {
        int contentIndex = 0;
        int elementCounter = 0;
        Iterator it = parent.getContent().iterator();
        Text lastText = null;
        while (it.hasNext() && elementCounter < counter.getCurrentIndex()) {
            Object next = it.next();
            if (next instanceof Element) {
                elementCounter = elementCounter + 1;
            }
            if (next instanceof Text) {
                lastText = (Text)next;
            }
            contentIndex = contentIndex + 1;
        }
        if (lastText != null && lastText.getTextTrim().length() == 0) {
            lastText = (Text)lastText.clone();
        } else {
            String starter = lineSeparator;
            Element curPar = parent;
            while (curPar != null) {
                curPar = curPar.getParentElement();
                starter = starter + "    "; //TODO make settable?
            }
            lastText = factory.text(starter);
        }
        if (counter.getCurrentIndex() >= elementCounter) {
            parent.addContent(lastText);
            parent.addContent(child);
        } else {
            parent.addContent(contentIndex, child);
            parent.addContent(contentIndex, lastText);
        }
    } //-- void insertAtPreferredLocation(Element, Element, Counter) 

    /**
     * Method iterateContributor
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateContributor(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "contributors", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("contributor", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Contributor value = (Contributor) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("contributor", element.getNamespace());
                    element.addContent(el);
                }
                updateContributor(value, "contributor", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateContributor(Counter, Element, java.util.Collection) 

    /**
     * Method iterateDependency
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateDependency(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "dependencies", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("dependency", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Dependency value = (Dependency) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("dependency", element.getNamespace());
                    element.addContent(el);
                }
                updateDependency(value, "dependency", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateDependency(Counter, Element, java.util.Collection) 

    /**
     * Method iterateDeveloper
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateDeveloper(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "developers", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("developer", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Developer value = (Developer) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("developer", element.getNamespace());
                    element.addContent(el);
                }
                updateDeveloper(value, "developer", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateDeveloper(Counter, Element, java.util.Collection) 

    /**
     * Method iterateExclusion
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateExclusion(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "exclusions", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("exclusion", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Exclusion value = (Exclusion) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("exclusion", element.getNamespace());
                    element.addContent(el);
                }
                updateExclusion(value, "exclusion", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateExclusion(Counter, Element, java.util.Collection) 

    /**
     * Method iterateExtension
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateExtension(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "extensions", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("extension", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Extension value = (Extension) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("extension", element.getNamespace());
                    element.addContent(el);
                }
                updateExtension(value, "extension", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateExtension(Counter, Element, java.util.Collection) 

    /**
     * Method iterateLicense
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateLicense(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "licenses", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("license", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                License value = (License) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("license", element.getNamespace());
                    element.addContent(el);
                }
                updateLicense(value, "license", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateLicense(Counter, Element, java.util.Collection) 

    /**
     * Method iterateMailingList
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateMailingList(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "mailingLists", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("mailingList", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                MailingList value = (MailingList) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("mailingList", element.getNamespace());
                    element.addContent(el);
                }
                updateMailingList(value, "mailingList", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateMailingList(Counter, Element, java.util.Collection) 

    /**
     * Method iterateNotifier
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateNotifier(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "notifiers", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("notifier", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Notifier value = (Notifier) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("notifier", element.getNamespace());
                    element.addContent(el);
                }
                updateNotifier(value, "notifier", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateNotifier(Counter, Element, java.util.Collection) 

    /**
     * Method iteratePlugin
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iteratePlugin(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "plugins", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("plugin", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Plugin value = (Plugin) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("plugin", element.getNamespace());
                    element.addContent(el);
                }
                updatePlugin(value, "plugin", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iteratePlugin(Counter, Element, java.util.Collection) 

    /**
     * Method iteratePluginExecution
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iteratePluginExecution(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "executions", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("execution", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                PluginExecution value = (PluginExecution) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("execution", element.getNamespace());
                    element.addContent(el);
                }
                updatePluginExecution(value, "execution", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iteratePluginExecution(Counter, Element, java.util.Collection) 

    /**
     * Method iterateProfile
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateProfile(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "profiles", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("profile", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Profile value = (Profile) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("profile", element.getNamespace());
                    element.addContent(el);
                }
                updateProfile(value, "profile", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateProfile(Counter, Element, java.util.Collection) 

    /**
     * Method iterateReportPlugin
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateReportPlugin(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "plugins", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("plugin", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                ReportPlugin value = (ReportPlugin) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("plugin", element.getNamespace());
                    element.addContent(el);
                }
                updateReportPlugin(value, "plugin", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateReportPlugin(Counter, Element, java.util.Collection) 

    /**
     * Method iterateReportSet
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateReportSet(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "reportSets", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("reportSet", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                ReportSet value = (ReportSet) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("reportSet", element.getNamespace());
                    element.addContent(el);
                }
                updateReportSet(value, "reportSet", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateReportSet(Counter, Element, java.util.Collection) 

    /**
     * Method iterateRepository
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateRepository(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "repositories", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("repository", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Repository value = (Repository) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("repository", element.getNamespace());
                    element.addContent(el);
                }
                updateRepository(value, "repository", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateRepository(Counter, Element, java.util.Collection) 

    /**
     * Method iterateResource
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateResource(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "resources", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("resource", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Resource value = (Resource) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("resource", element.getNamespace());
                    element.addContent(el);
                }
                updateResource(value, "resource", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateResource(Counter, Element, java.util.Collection) 

    /**
     * Method replaceXpp3DOM
     * 
     * @param parent
     * @param parentDom
     */
    public void replaceXpp3DOM(Element parent, Xpp3Dom parentDom)
    {
        if (parentDom.getChildCount() > 0) {
            Xpp3Dom[] childs = parentDom.getChildren();
            Collection domChilds = new ArrayList();
            for (int i = 0; i < childs.length; i++) {
                domChilds.add(childs[i]);
            }
            int domIndex = 0;
            ListIterator it = parent.getChildren().listIterator();
            while (it.hasNext()) {
                Element elem = (Element) it.next();
                Iterator it2 = domChilds.iterator();
                Xpp3Dom corrDom = null;
                while (it2.hasNext()) {
                    Xpp3Dom dm = (Xpp3Dom)it2.next();
                    if (dm.getName().equals(elem.getName())) {
                        corrDom = dm;
                        break;
                    }
                }
                if (corrDom != null) {
                    domChilds.remove(corrDom);
                    replaceXpp3DOM(elem, corrDom);
                } else {
                    parent.removeContent(elem);
                }
            }
            Iterator it2 = domChilds.iterator();
            while (it2.hasNext()) {
                Xpp3Dom dm = (Xpp3Dom) it2.next();
                Element elem = factory.element(dm.getName(), parent.getNamespace());
                parent.addContent(elem);
                replaceXpp3DOM(elem, dm);
            }
        } else if (parentDom.getValue() != null) {
            parent.setText(parentDom.getValue());
        }
    } //-- void replaceXpp3DOM(Element, Xpp3Dom) 

    /**
     * Method updateActivation
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateActivation(Activation value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "activeByDefault", value.isActiveByDefault() == false ? null : String.valueOf( value.isActiveByDefault() ));
            findAndReplaceSimpleElement(innerCount, root,  "jdk", value.getJdk());
            updateActivationOS( value.getOs(), "os", innerCount, root);
            updateActivationProperty( value.getProperty(), "property", innerCount, root);
            updateActivationFile( value.getFile(), "file", innerCount, root);
        }
    } //-- void updateActivation(Activation, String, Counter, Element) 

    /**
     * Method updateActivationFile
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateActivationFile(ActivationFile value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "missing", value.getMissing());
            findAndReplaceSimpleElement(innerCount, root,  "exists", value.getExists());
        }
    } //-- void updateActivationFile(ActivationFile, String, Counter, Element) 

    /**
     * Method updateActivationOS
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateActivationOS(ActivationOS value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
            findAndReplaceSimpleElement(innerCount, root,  "family", value.getFamily());
            findAndReplaceSimpleElement(innerCount, root,  "arch", value.getArch());
            findAndReplaceSimpleElement(innerCount, root,  "version", value.getVersion());
        }
    } //-- void updateActivationOS(ActivationOS, String, Counter, Element) 

    /**
     * Method updateActivationProperty
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateActivationProperty(ActivationProperty value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
            findAndReplaceSimpleElement(innerCount, root,  "value", value.getValue());
        }
    } //-- void updateActivationProperty(ActivationProperty, String, Counter, Element) 

    /**
     * Method updateBuild
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateBuild(Build value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "sourceDirectory", value.getSourceDirectory());
            findAndReplaceSimpleElement(innerCount, root,  "scriptSourceDirectory", value.getScriptSourceDirectory());
            findAndReplaceSimpleElement(innerCount, root,  "testSourceDirectory", value.getTestSourceDirectory());
            findAndReplaceSimpleElement(innerCount, root,  "outputDirectory", value.getOutputDirectory());
            findAndReplaceSimpleElement(innerCount, root,  "testOutputDirectory", value.getTestOutputDirectory());
            iterateExtension(innerCount, root, value.getExtensions());
            findAndReplaceSimpleElement(innerCount, root,  "defaultGoal", value.getDefaultGoal());
            iterateResource(innerCount, root, value.getResources());
            iterateResource(innerCount, root, value.getTestResources());
            findAndReplaceSimpleElement(innerCount, root,  "directory", value.getDirectory());
            findAndReplaceSimpleElement(innerCount, root,  "finalName", value.getFinalName());
            findAndReplaceSimpleLists(innerCount, root, value.getFilters(), "filters", "filter");
            updatePluginManagement( value.getPluginManagement(), "pluginManagement", innerCount, root);
            iteratePlugin(innerCount, root, value.getPlugins());
        }
    } //-- void updateBuild(Build, String, Counter, Element) 

    /**
     * Method updateBuildBase
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateBuildBase(BuildBase value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "defaultGoal", value.getDefaultGoal());
            iterateResource(innerCount, root, value.getResources());
            iterateResource(innerCount, root, value.getTestResources());
            findAndReplaceSimpleElement(innerCount, root,  "directory", value.getDirectory());
            findAndReplaceSimpleElement(innerCount, root,  "finalName", value.getFinalName());
            findAndReplaceSimpleLists(innerCount, root, value.getFilters(), "filters", "filter");
            updatePluginManagement( value.getPluginManagement(), "pluginManagement", innerCount, root);
            iteratePlugin(innerCount, root, value.getPlugins());
        }
    } //-- void updateBuildBase(BuildBase, String, Counter, Element) 

    /**
     * Method updateCiManagement
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateCiManagement(CiManagement value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "system", value.getSystem());
            findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
            iterateNotifier(innerCount, root, value.getNotifiers());
        }
    } //-- void updateCiManagement(CiManagement, String, Counter, Element) 

    /**
     * Method updateConfigurationContainer
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateConfigurationContainer(ConfigurationContainer value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "inherited", value.getInherited());
            findAndReplaceXpp3DOM(innerCount, root, "configuration", (Xpp3Dom)value.getConfiguration());
        }
    } //-- void updateConfigurationContainer(ConfigurationContainer, String, Counter, Element) 

    /**
     * Method updateContributor
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateContributor(Contributor value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
        findAndReplaceSimpleElement(innerCount, root,  "email", value.getEmail());
        findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
        findAndReplaceSimpleElement(innerCount, root,  "organization", value.getOrganization());
        findAndReplaceSimpleElement(innerCount, root,  "organizationUrl", value.getOrganizationUrl());
        findAndReplaceSimpleLists(innerCount, root, value.getRoles(), "roles", "role");
        findAndReplaceSimpleElement(innerCount, root,  "timezone", value.getTimezone());
        findAndReplaceProperties(innerCount, root,  "properties", value.getProperties());
    } //-- void updateContributor(Contributor, String, Counter, Element) 

    /**
     * Method updateDependency
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateDependency(Dependency value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "groupId", value.getGroupId());
        findAndReplaceSimpleElement(innerCount, root,  "artifactId", value.getArtifactId());
        findAndReplaceSimpleElement(innerCount, root,  "version", value.getVersion());
        findAndReplaceSimpleElement(innerCount, root,  "type", value.getType() == null || value.getType().equals( "jar" ) ? null : value.getType());
        findAndReplaceSimpleElement(innerCount, root,  "classifier", value.getClassifier());
        findAndReplaceSimpleElement(innerCount, root,  "scope", value.getScope());
        findAndReplaceSimpleElement(innerCount, root,  "systemPath", value.getSystemPath());
        iterateExclusion(innerCount, root, value.getExclusions());
        findAndReplaceSimpleElement(innerCount, root,  "optional", value.isOptional() == false ? null : String.valueOf( value.isOptional() ));
    } //-- void updateDependency(Dependency, String, Counter, Element) 

    /**
     * Method updateDependencyManagement
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateDependencyManagement(DependencyManagement value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            iterateDependency(innerCount, root, value.getDependencies());
        }
    } //-- void updateDependencyManagement(DependencyManagement, String, Counter, Element) 

    /**
     * Method updateDeploymentRepository
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateDeploymentRepository(DeploymentRepository value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "uniqueVersion", value.isUniqueVersion() == true ? null : String.valueOf( value.isUniqueVersion() ));
            findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
            findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
            findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
            findAndReplaceSimpleElement(innerCount, root,  "layout", value.getLayout() == null || value.getLayout().equals( "default" ) ? null : value.getLayout());
        }
    } //-- void updateDeploymentRepository(DeploymentRepository, String, Counter, Element) 

    /**
     * Method updateDeveloper
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateDeveloper(Developer value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
        findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
        findAndReplaceSimpleElement(innerCount, root,  "email", value.getEmail());
        findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
        findAndReplaceSimpleElement(innerCount, root,  "organization", value.getOrganization());
        findAndReplaceSimpleElement(innerCount, root,  "organizationUrl", value.getOrganizationUrl());
        findAndReplaceSimpleLists(innerCount, root, value.getRoles(), "roles", "role");
        findAndReplaceSimpleElement(innerCount, root,  "timezone", value.getTimezone());
        findAndReplaceProperties(innerCount, root,  "properties", value.getProperties());
    } //-- void updateDeveloper(Developer, String, Counter, Element) 

    /**
     * Method updateDistributionManagement
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateDistributionManagement(DistributionManagement value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            updateDeploymentRepository( value.getRepository(), "repository", innerCount, root);
            updateDeploymentRepository( value.getSnapshotRepository(), "snapshotRepository", innerCount, root);
            updateSite( value.getSite(), "site", innerCount, root);
            findAndReplaceSimpleElement(innerCount, root,  "downloadUrl", value.getDownloadUrl());
            updateRelocation( value.getRelocation(), "relocation", innerCount, root);
            findAndReplaceSimpleElement(innerCount, root,  "status", value.getStatus());
        }
    } //-- void updateDistributionManagement(DistributionManagement, String, Counter, Element) 

    /**
     * Method updateElement
     * 
     * @param counter
     * @param shouldExist
     * @param name
     * @param parent
     */
    public Element updateElement(Counter counter, Element parent, String name, boolean shouldExist)
    {
        Element element =  parent.getChild(name, parent.getNamespace());
        if (element != null && shouldExist) {
            if (parent.getChildren().indexOf(element) <= counter.getCurrentIndex()) {
                counter.increaseCount();
            }
        }
        if (element == null && shouldExist) {
            element = factory.element(name, parent.getNamespace());
            insertAtPreferredLocation(parent, element, counter);
            counter.increaseCount();
        }
        if (!shouldExist && element != null) {
            parent.removeChild(name, parent.getNamespace());
        }
        return element;
    } //-- Element updateElement(Counter, Element, String, boolean) 

    /**
     * Method updateExclusion
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateExclusion(Exclusion value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "artifactId", value.getArtifactId());
        findAndReplaceSimpleElement(innerCount, root,  "groupId", value.getGroupId());
    } //-- void updateExclusion(Exclusion, String, Counter, Element) 

    /**
     * Method updateExtension
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateExtension(Extension value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "groupId", value.getGroupId());
        findAndReplaceSimpleElement(innerCount, root,  "artifactId", value.getArtifactId());
        findAndReplaceSimpleElement(innerCount, root,  "version", value.getVersion());
    } //-- void updateExtension(Extension, String, Counter, Element) 

    /**
     * Method updateFileSet
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateFileSet(FileSet value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "directory", value.getDirectory());
            findAndReplaceSimpleLists(innerCount, root, value.getIncludes(), "includes", "include");
            findAndReplaceSimpleLists(innerCount, root, value.getExcludes(), "excludes", "exclude");
        }
    } //-- void updateFileSet(FileSet, String, Counter, Element) 

    /**
     * Method updateIssueManagement
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateIssueManagement(IssueManagement value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "system", value.getSystem());
            findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
        }
    } //-- void updateIssueManagement(IssueManagement, String, Counter, Element) 

    /**
     * Method updateLicense
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateLicense(License value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
        findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
        findAndReplaceSimpleElement(innerCount, root,  "distribution", value.getDistribution());
        findAndReplaceSimpleElement(innerCount, root,  "comments", value.getComments());
    } //-- void updateLicense(License, String, Counter, Element) 

    /**
     * Method updateMailingList
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateMailingList(MailingList value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
        findAndReplaceSimpleElement(innerCount, root,  "subscribe", value.getSubscribe());
        findAndReplaceSimpleElement(innerCount, root,  "unsubscribe", value.getUnsubscribe());
        findAndReplaceSimpleElement(innerCount, root,  "post", value.getPost());
        findAndReplaceSimpleElement(innerCount, root,  "archive", value.getArchive());
        findAndReplaceSimpleLists(innerCount, root, value.getOtherArchives(), "otherArchives", "otherArchive");
    } //-- void updateMailingList(MailingList, String, Counter, Element) 

    /**
     * Method updateModel
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateModel(Model value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        updateParent( value.getParent(), "parent", innerCount, root);
        findAndReplaceSimpleElement(innerCount, root,  "modelVersion", value.getModelVersion());
        findAndReplaceSimpleElement(innerCount, root,  "groupId", value.getGroupId());
        findAndReplaceSimpleElement(innerCount, root,  "artifactId", value.getArtifactId());
        findAndReplaceSimpleElement(innerCount, root,  "packaging", value.getPackaging() == null || value.getPackaging().equals( "jar" ) ? null : value.getPackaging());
        findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
        findAndReplaceSimpleElement(innerCount, root,  "version", value.getVersion());
        findAndReplaceSimpleElement(innerCount, root,  "description", value.getDescription());
        findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
        updatePrerequisites( value.getPrerequisites(), "prerequisites", innerCount, root);
        updateIssueManagement( value.getIssueManagement(), "issueManagement", innerCount, root);
        updateCiManagement( value.getCiManagement(), "ciManagement", innerCount, root);
        findAndReplaceSimpleElement(innerCount, root,  "inceptionYear", value.getInceptionYear());
        iterateMailingList(innerCount, root, value.getMailingLists());
        iterateDeveloper(innerCount, root, value.getDevelopers());
        iterateContributor(innerCount, root, value.getContributors());
        iterateLicense(innerCount, root, value.getLicenses());
        updateScm( value.getScm(), "scm", innerCount, root);
        updateOrganization( value.getOrganization(), "organization", innerCount, root);
        updateBuild( value.getBuild(), "build", innerCount, root);
        iterateProfile(innerCount, root, value.getProfiles());
        findAndReplaceSimpleLists(innerCount, root, value.getModules(), "modules", "module");
        iterateRepository(innerCount, root, value.getRepositories());
        iterateRepository(innerCount, root, value.getPluginRepositories());
        iterateDependency(innerCount, root, value.getDependencies());
        findAndReplaceXpp3DOM(innerCount, root, "reports", (Xpp3Dom)value.getReports());
        updateReporting( value.getReporting(), "reporting", innerCount, root);
        updateDependencyManagement( value.getDependencyManagement(), "dependencyManagement", innerCount, root);
        updateDistributionManagement( value.getDistributionManagement(), "distributionManagement", innerCount, root);
        findAndReplaceProperties(innerCount, root,  "properties", value.getProperties());
    } //-- void updateModel(Model, String, Counter, Element) 

    /**
     * Method updateModelBase
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateModelBase(ModelBase value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleLists(innerCount, root, value.getModules(), "modules", "module");
            iterateRepository(innerCount, root, value.getRepositories());
            iterateRepository(innerCount, root, value.getPluginRepositories());
            iterateDependency(innerCount, root, value.getDependencies());
            findAndReplaceXpp3DOM(innerCount, root, "reports", (Xpp3Dom)value.getReports());
            updateReporting( value.getReporting(), "reporting", innerCount, root);
            updateDependencyManagement( value.getDependencyManagement(), "dependencyManagement", innerCount, root);
            updateDistributionManagement( value.getDistributionManagement(), "distributionManagement", innerCount, root);
            findAndReplaceProperties(innerCount, root,  "properties", value.getProperties());
        }
    } //-- void updateModelBase(ModelBase, String, Counter, Element) 

    /**
     * Method updateNotifier
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateNotifier(Notifier value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "type", value.getType() == null || value.getType().equals( "mail" ) ? null : value.getType());
        findAndReplaceSimpleElement(innerCount, root,  "sendOnError", value.isSendOnError() == true ? null : String.valueOf( value.isSendOnError() ));
        findAndReplaceSimpleElement(innerCount, root,  "sendOnFailure", value.isSendOnFailure() == true ? null : String.valueOf( value.isSendOnFailure() ));
        findAndReplaceSimpleElement(innerCount, root,  "sendOnSuccess", value.isSendOnSuccess() == true ? null : String.valueOf( value.isSendOnSuccess() ));
        findAndReplaceSimpleElement(innerCount, root,  "sendOnWarning", value.isSendOnWarning() == true ? null : String.valueOf( value.isSendOnWarning() ));
        findAndReplaceSimpleElement(innerCount, root,  "address", value.getAddress());
        findAndReplaceProperties(innerCount, root,  "configuration", value.getConfiguration());
    } //-- void updateNotifier(Notifier, String, Counter, Element) 

    /**
     * Method updateOrganization
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateOrganization(Organization value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
            findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
        }
    } //-- void updateOrganization(Organization, String, Counter, Element) 

    /**
     * Method updateParent
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateParent(Parent value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "artifactId", value.getArtifactId());
            findAndReplaceSimpleElement(innerCount, root,  "groupId", value.getGroupId());
            findAndReplaceSimpleElement(innerCount, root,  "version", value.getVersion());
            findAndReplaceSimpleElement(innerCount, root,  "relativePath", value.getRelativePath() == null || value.getRelativePath().equals( "../pom.xml" ) ? null : value.getRelativePath());
        }
    } //-- void updateParent(Parent, String, Counter, Element) 

    /**
     * Method updatePatternSet
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updatePatternSet(PatternSet value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleLists(innerCount, root, value.getIncludes(), "includes", "include");
            findAndReplaceSimpleLists(innerCount, root, value.getExcludes(), "excludes", "exclude");
        }
    } //-- void updatePatternSet(PatternSet, String, Counter, Element) 

    /**
     * Method updatePlugin
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updatePlugin(Plugin value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "groupId", value.getGroupId() == null || value.getGroupId().equals( "org.apache.maven.plugins" ) ? null : value.getGroupId());
        findAndReplaceSimpleElement(innerCount, root,  "artifactId", value.getArtifactId());
        findAndReplaceSimpleElement(innerCount, root,  "version", value.getVersion());
        findAndReplaceSimpleElement(innerCount, root,  "extensions", value.isExtensions() == false ? null : String.valueOf( value.isExtensions() ));
        iteratePluginExecution(innerCount, root, value.getExecutions());
        iterateDependency(innerCount, root, value.getDependencies());
        findAndReplaceXpp3DOM(innerCount, root, "goals", (Xpp3Dom)value.getGoals());
        findAndReplaceSimpleElement(innerCount, root,  "inherited", value.getInherited());
        findAndReplaceXpp3DOM(innerCount, root, "configuration", (Xpp3Dom)value.getConfiguration());
    } //-- void updatePlugin(Plugin, String, Counter, Element) 

    /**
     * Method updatePluginConfiguration
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updatePluginConfiguration(PluginConfiguration value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            updatePluginManagement( value.getPluginManagement(), "pluginManagement", innerCount, root);
            iteratePlugin(innerCount, root, value.getPlugins());
        }
    } //-- void updatePluginConfiguration(PluginConfiguration, String, Counter, Element) 

    /**
     * Method updatePluginContainer
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updatePluginContainer(PluginContainer value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            iteratePlugin(innerCount, root, value.getPlugins());
        }
    } //-- void updatePluginContainer(PluginContainer, String, Counter, Element) 

    /**
     * Method updatePluginExecution
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updatePluginExecution(PluginExecution value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "id", value.getId() == null || value.getId().equals( "default" ) ? null : value.getId());
        findAndReplaceSimpleElement(innerCount, root,  "phase", value.getPhase());
        findAndReplaceSimpleLists(innerCount, root, value.getGoals(), "goals", "goal");
        findAndReplaceSimpleElement(innerCount, root,  "inherited", value.getInherited());
        findAndReplaceXpp3DOM(innerCount, root, "configuration", (Xpp3Dom)value.getConfiguration());
    } //-- void updatePluginExecution(PluginExecution, String, Counter, Element) 

    /**
     * Method updatePluginManagement
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updatePluginManagement(PluginManagement value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            iteratePlugin(innerCount, root, value.getPlugins());
        }
    } //-- void updatePluginManagement(PluginManagement, String, Counter, Element) 

    /**
     * Method updatePrerequisites
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updatePrerequisites(Prerequisites value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "maven", value.getMaven() == null || value.getMaven().equals( "2.0" ) ? null : value.getMaven());
        }
    } //-- void updatePrerequisites(Prerequisites, String, Counter, Element) 

    /**
     * Method updateProfile
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateProfile(Profile value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
        updateActivation( value.getActivation(), "activation", innerCount, root);
        updateBuildBase( value.getBuild(), "build", innerCount, root);
        findAndReplaceSimpleLists(innerCount, root, value.getModules(), "modules", "module");
        iterateRepository(innerCount, root, value.getRepositories());
        iterateRepository(innerCount, root, value.getPluginRepositories());
        iterateDependency(innerCount, root, value.getDependencies());
        findAndReplaceXpp3DOM(innerCount, root, "reports", (Xpp3Dom)value.getReports());
        updateReporting( value.getReporting(), "reporting", innerCount, root);
        updateDependencyManagement( value.getDependencyManagement(), "dependencyManagement", innerCount, root);
        updateDistributionManagement( value.getDistributionManagement(), "distributionManagement", innerCount, root);
        findAndReplaceProperties(innerCount, root,  "properties", value.getProperties());
    } //-- void updateProfile(Profile, String, Counter, Element) 

    /**
     * Method updateRelocation
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateRelocation(Relocation value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "groupId", value.getGroupId());
            findAndReplaceSimpleElement(innerCount, root,  "artifactId", value.getArtifactId());
            findAndReplaceSimpleElement(innerCount, root,  "version", value.getVersion());
            findAndReplaceSimpleElement(innerCount, root,  "message", value.getMessage());
        }
    } //-- void updateRelocation(Relocation, String, Counter, Element) 

    /**
     * Method updateReportPlugin
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateReportPlugin(ReportPlugin value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "groupId", value.getGroupId() == null || value.getGroupId().equals( "org.apache.maven.plugins" ) ? null : value.getGroupId());
        findAndReplaceSimpleElement(innerCount, root,  "artifactId", value.getArtifactId());
        findAndReplaceSimpleElement(innerCount, root,  "version", value.getVersion());
        findAndReplaceSimpleElement(innerCount, root,  "inherited", value.getInherited());
        findAndReplaceXpp3DOM(innerCount, root, "configuration", (Xpp3Dom)value.getConfiguration());
        iterateReportSet(innerCount, root, value.getReportSets());
    } //-- void updateReportPlugin(ReportPlugin, String, Counter, Element) 

    /**
     * Method updateReportSet
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateReportSet(ReportSet value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "id", value.getId() == null || value.getId().equals( "default" ) ? null : value.getId());
        findAndReplaceXpp3DOM(innerCount, root, "configuration", (Xpp3Dom)value.getConfiguration());
        findAndReplaceSimpleElement(innerCount, root,  "inherited", value.getInherited());
        findAndReplaceSimpleLists(innerCount, root, value.getReports(), "reports", "report");
    } //-- void updateReportSet(ReportSet, String, Counter, Element) 

    /**
     * Method updateReporting
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateReporting(Reporting value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "excludeDefaults", value.isExcludeDefaults() == false ? null : String.valueOf( value.isExcludeDefaults() ));
            findAndReplaceSimpleElement(innerCount, root,  "outputDirectory", value.getOutputDirectory());
            iterateReportPlugin(innerCount, root, value.getPlugins());
        }
    } //-- void updateReporting(Reporting, String, Counter, Element) 

    /**
     * Method updateRepository
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateRepository(Repository value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        updateRepositoryPolicy( value.getReleases(), "releases", innerCount, root);
        updateRepositoryPolicy( value.getSnapshots(), "snapshots", innerCount, root);
        findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
        findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
        findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
        findAndReplaceSimpleElement(innerCount, root,  "layout", value.getLayout() == null || value.getLayout().equals( "default" ) ? null : value.getLayout());
    } //-- void updateRepository(Repository, String, Counter, Element) 

    /**
     * Method updateRepositoryBase
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateRepositoryBase(RepositoryBase value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
            findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
            findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
            findAndReplaceSimpleElement(innerCount, root,  "layout", value.getLayout() == null || value.getLayout().equals( "default" ) ? null : value.getLayout());
        }
    } //-- void updateRepositoryBase(RepositoryBase, String, Counter, Element) 

    /**
     * Method updateRepositoryPolicy
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateRepositoryPolicy(RepositoryPolicy value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "enabled", value.isEnabled() == true ? null : String.valueOf( value.isEnabled() ));
            findAndReplaceSimpleElement(innerCount, root,  "updatePolicy", value.getUpdatePolicy());
            findAndReplaceSimpleElement(innerCount, root,  "checksumPolicy", value.getChecksumPolicy());
        }
    } //-- void updateRepositoryPolicy(RepositoryPolicy, String, Counter, Element) 

    /**
     * Method updateResource
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateResource(Resource value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "targetPath", value.getTargetPath());
        findAndReplaceSimpleElement(innerCount, root,  "filtering", value.isFiltering() == false ? null : String.valueOf( value.isFiltering() ));
        findAndReplaceSimpleElement(innerCount, root,  "directory", value.getDirectory());
        findAndReplaceSimpleLists(innerCount, root, value.getIncludes(), "includes", "include");
        findAndReplaceSimpleLists(innerCount, root, value.getExcludes(), "excludes", "exclude");
    } //-- void updateResource(Resource, String, Counter, Element) 

    /**
     * Method updateScm
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateScm(Scm value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "connection", value.getConnection());
            findAndReplaceSimpleElement(innerCount, root,  "developerConnection", value.getDeveloperConnection());
            findAndReplaceSimpleElement(innerCount, root,  "tag", value.getTag() == null || value.getTag().equals( "HEAD" ) ? null : value.getTag());
            findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
        }
    } //-- void updateScm(Scm, String, Counter, Element) 

    /**
     * Method updateSite
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateSite(Site value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
            findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
            findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
        }
    } //-- void updateSite(Site, String, Counter, Element) 

    /**
     * Method write
     * 
     * @param project
     * @param stream
     * @param document
     */
    public void write(Model project, Document document, OutputStream stream)
        throws java.io.IOException
    {
        updateModel(project, "project", new Counter(), document.getRootElement());
        XMLOutputter outputter = new XMLOutputter();
        outputter.setFormat(Format.getPrettyFormat()
        .setIndent("    ")
        .setLineSeparator(System.getProperty("line.separator")));
        outputter.output(document, stream);
    } //-- void write(Model, Document, OutputStream) 


      //-----------------/
     //- Inner Classes -/
    //-----------------/

    /**
     * Class Counter.
     * 
     * @version $Revision$ $Date$
     */
    public class Counter {


          //--------------------------/
         //- Class/Member Variables -/
        //--------------------------/

        /**
         * Field currentIndex
         */
        private int currentIndex = 0;


          //-----------/
         //- Methods -/
        //-----------/

        /**
         * Method getCurrentIndex
         */
        public int getCurrentIndex()
        {
            return currentIndex;
        } //-- int getCurrentIndex() 

        /**
         * Method increaseCount
         */
        public void increaseCount()
        {
            currentIndex = currentIndex + 1;
        } //-- void increaseCount() 

    }

}
