/*
 * $Id$
 */

package org.apache.maven.settings.io.jdom;

  //---------------------------------/
 //- Imported classes and packages -/
//---------------------------------/

import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Properties;
import org.apache.maven.model.Model;
import org.apache.maven.model.Parent;
import org.apache.maven.settings.Activation;
import org.apache.maven.settings.ActivationFile;
import org.apache.maven.settings.ActivationOS;
import org.apache.maven.settings.ActivationProperty;
import org.apache.maven.settings.IdentifiableBase;
import org.apache.maven.settings.Mirror;
import org.apache.maven.settings.Profile;
import org.apache.maven.settings.Proxy;
import org.apache.maven.settings.Repository;
import org.apache.maven.settings.RepositoryBase;
import org.apache.maven.settings.RepositoryPolicy;
import org.apache.maven.settings.Server;
import org.apache.maven.settings.Settings;
import org.apache.maven.settings.TrackableBase;
import org.codehaus.plexus.util.xml.Xpp3Dom;
import org.jdom.Content;
import org.jdom.DefaultJDOMFactory;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Text;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * Class SettingsJDOMWriter.
 * 
 * @version $Revision$ $Date$
 */
public class SettingsJDOMWriter {


      //--------------------------/
     //- Class/Member Variables -/
    //--------------------------/

    /**
     * Field factory
     */
    private DefaultJDOMFactory factory;

    /**
     * Field lineSeparator
     */
    private String lineSeparator;


      //----------------/
     //- Constructors -/
    //----------------/

    public SettingsJDOMWriter() {
        factory = new DefaultJDOMFactory();
        lineSeparator = System.getProperty("line.separator");
    } //-- org.apache.maven.settings.io.jdom.SettingsJDOMWriter()


      //-----------/
     //- Methods -/
    //-----------/

    /**
     * Method findAndReplaceProperties
     * 
     * @param counter
     * @param props
     * @param name
     * @param parent
     */
    public Element findAndReplaceProperties(Counter counter, Element parent, String name, Map props)
    {
        boolean shouldExist = props != null && ! props.isEmpty();
        Element element = updateElement(counter, parent, name, shouldExist);
        if (shouldExist) {
            Iterator it = props.keySet().iterator();
            Counter innerCounter = new Counter();
            while (it.hasNext()) {
                String key = (String) it.next();
                findAndReplaceSimpleElement(innerCounter, element, key, (String)props.get(key));
                }
            ArrayList lst = new ArrayList(props.keySet());
            it = element.getChildren().iterator();
            while (it.hasNext()) {
                Element elem = (Element) it.next();
                String key = elem.getName();
                if (!lst.contains(key)) {
                    it.remove();
                }
            }
        }
        return element;
    } //-- Element findAndReplaceProperties(Counter, Element, String, Map) 

    /**
     * Method findAndReplaceSimpleElement
     * 
     * @param counter
     * @param text
     * @param name
     * @param parent
     */
    public Element findAndReplaceSimpleElement(Counter counter, Element parent, String name, String text)
    {
        boolean shouldExist = text != null && text.trim().length() > 0;
        Element element = updateElement(counter, parent, name, shouldExist);
        if (shouldExist) {
            element.setText(text);
        }
        return element;
    } //-- Element findAndReplaceSimpleElement(Counter, Element, String, String) 

    /**
     * Method findAndReplaceSimpleLists
     * 
     * @param counter
     * @param childName
     * @param parentName
     * @param list
     * @param parent
     */
    public Element findAndReplaceSimpleLists(Counter counter, Element parent, java.util.Collection list, String parentName, String childName)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, parentName, shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren(childName, element.getNamespace()).iterator();
            if (! elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                String value = (String) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element(childName, element.getNamespace());
                    element.addContent(el);
                }
                el.setText(value);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
        return element;
    } //-- Element findAndReplaceSimpleLists(Counter, Element, java.util.Collection, String, String) 

    /**
     * Method findAndReplaceXpp3DOM
     * 
     * @param counter
     * @param dom
     * @param name
     * @param parent
     */
    public Element findAndReplaceXpp3DOM(Counter counter, Element parent, String name, Xpp3Dom dom)
    {
        boolean shouldExist = dom != null && (dom.getChildCount() > 0 || dom.getValue() != null);
        Element element = updateElement(counter, parent, name, shouldExist);
        if (shouldExist) {
            replaceXpp3DOM(element, dom);
        }
        return element;
    } //-- Element findAndReplaceXpp3DOM(Counter, Element, String, Xpp3Dom) 

    /**
     * Method insertAtPreferredLocation
     * 
     * @param parent
     * @param counter
     * @param child
     */
    public void insertAtPreferredLocation(Element parent, Element child, Counter counter)
    {
        int contentIndex = 0;
        int elementCounter = 0;
        Iterator it = parent.getContent().iterator();
        Text lastText = null;
        while (it.hasNext() && elementCounter < counter.getCurrentIndex()) {
            Object next = it.next();
            if (next instanceof Element) {
                elementCounter = elementCounter + 1;
            }
            if (next instanceof Text) {
                lastText = (Text)next;
            }
            contentIndex = contentIndex + 1;
        }
        if (lastText != null && lastText.getTextTrim().length() == 0) {
            lastText = (Text)lastText.clone();
        } else {
            String starter = lineSeparator;
            Element curPar = parent;
            while (curPar != null) {
                curPar = curPar.getParentElement();
                starter = starter + "    "; //TODO make settable?
            }
            lastText = factory.text(starter);
        }
        if (counter.getCurrentIndex() >= elementCounter) {
            parent.addContent(lastText);
            parent.addContent(child);
        } else {
            parent.addContent(contentIndex, child);
            parent.addContent(contentIndex, lastText);
        }
    } //-- void insertAtPreferredLocation(Element, Element, Counter) 

    /**
     * Method iterateMirror
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateMirror(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "mirrors", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("mirror", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Mirror value = (Mirror) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("mirror", element.getNamespace());
                    element.addContent(el);
                }
                updateMirror(value, "mirror", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateMirror(Counter, Element, java.util.Collection) 

    /**
     * Method iterateProfile
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateProfile(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "profiles", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("profile", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Profile value = (Profile) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("profile", element.getNamespace());
                    element.addContent(el);
                }
                updateProfile(value, "profile", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateProfile(Counter, Element, java.util.Collection) 

    /**
     * Method iterateProxy
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateProxy(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "proxies", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("proxy", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Proxy value = (Proxy) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("proxy", element.getNamespace());
                    element.addContent(el);
                }
                updateProxy(value, "proxy", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateProxy(Counter, Element, java.util.Collection) 

    /**
     * Method iterateRepository
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateRepository(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "repositories", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("repository", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Repository value = (Repository) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("repository", element.getNamespace());
                    element.addContent(el);
                }
                updateRepository(value, "repository", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateRepository(Counter, Element, java.util.Collection) 

    /**
     * Method iterateServer
     * 
     * @param counter
     * @param list
     * @param parent
     */
    public void iterateServer(Counter counter, Element parent, java.util.Collection list)
    {
        boolean shouldExist = list != null && list.size() > 0;
        Element element = updateElement(counter, parent, "servers", shouldExist);
        if (shouldExist) {
            Iterator it = list.iterator();
            Iterator elIt = element.getChildren("server", element.getNamespace()).iterator();
            if (!elIt.hasNext()) elIt = null;
            Counter innerCount = new Counter();
            while (it.hasNext()) {
                Server value = (Server) it.next();
                Element el;
                if (elIt != null && elIt.hasNext()) {
                    el = (Element) elIt.next();
                    if (! elIt.hasNext()) elIt = null;
                } else {
                    el = factory.element("server", element.getNamespace());
                    element.addContent(el);
                }
                updateServer(value, "server", innerCount, el);
            }
            if (elIt != null) {
                while (elIt.hasNext()) {
                    elIt.next();
                    elIt.remove();
                }
            }
        }
    } //-- void iterateServer(Counter, Element, java.util.Collection) 

    /**
     * Method replaceXpp3DOM
     * 
     * @param parent
     * @param parentDom
     */
    public void replaceXpp3DOM(Element parent, Xpp3Dom parentDom)
    {
        if (parentDom.getChildCount() > 0) {
            Xpp3Dom[] childs = parentDom.getChildren();
            Collection domChilds = new ArrayList();
            for (int i = 0; i < childs.length; i++) {
                domChilds.add(childs[i]);
            }
            int domIndex = 0;
            ListIterator it = parent.getChildren().listIterator();
            while (it.hasNext()) {
                Element elem = (Element) it.next();
                Iterator it2 = domChilds.iterator();
                Xpp3Dom corrDom = null;
                while (it2.hasNext()) {
                    Xpp3Dom dm = (Xpp3Dom)it2.next();
                    if (dm.getName().equals(elem.getName())) {
                        corrDom = dm;
                        break;
                    }
                }
                if (corrDom != null) {
                    domChilds.remove(corrDom);
                    replaceXpp3DOM(elem, corrDom);
                } else {
                    parent.removeContent(elem);
                }
            }
            Iterator it2 = domChilds.iterator();
            while (it2.hasNext()) {
                Xpp3Dom dm = (Xpp3Dom) it2.next();
                Element elem = factory.element(dm.getName(), parent.getNamespace());
                parent.addContent(elem);
                replaceXpp3DOM(elem, dm);
            }
        } else if (parentDom.getValue() != null) {
            parent.setText(parentDom.getValue());
        }
    } //-- void replaceXpp3DOM(Element, Xpp3Dom) 

    /**
     * Method updateActivation
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateActivation(Activation value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "activeByDefault", value.isActiveByDefault() == false ? null : String.valueOf( value.isActiveByDefault() ));
            findAndReplaceSimpleElement(innerCount, root,  "jdk", value.getJdk());
            updateActivationOS( value.getOs(), "os", innerCount, root);
            updateActivationProperty( value.getProperty(), "property", innerCount, root);
            updateActivationFile( value.getFile(), "file", innerCount, root);
        }
    } //-- void updateActivation(Activation, String, Counter, Element) 

    /**
     * Method updateActivationFile
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateActivationFile(ActivationFile value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "missing", value.getMissing());
            findAndReplaceSimpleElement(innerCount, root,  "exists", value.getExists());
        }
    } //-- void updateActivationFile(ActivationFile, String, Counter, Element) 

    /**
     * Method updateActivationOS
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateActivationOS(ActivationOS value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
            findAndReplaceSimpleElement(innerCount, root,  "family", value.getFamily());
            findAndReplaceSimpleElement(innerCount, root,  "arch", value.getArch());
            findAndReplaceSimpleElement(innerCount, root,  "version", value.getVersion());
        }
    } //-- void updateActivationOS(ActivationOS, String, Counter, Element) 

    /**
     * Method updateActivationProperty
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateActivationProperty(ActivationProperty value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
            findAndReplaceSimpleElement(innerCount, root,  "value", value.getValue());
        }
    } //-- void updateActivationProperty(ActivationProperty, String, Counter, Element) 

    /**
     * Method updateElement
     * 
     * @param counter
     * @param shouldExist
     * @param name
     * @param parent
     */
    public Element updateElement(Counter counter, Element parent, String name, boolean shouldExist)
    {
        Element element =  parent.getChild(name, parent.getNamespace());
        if (element != null && shouldExist) {
            if (parent.getChildren().indexOf(element) <= counter.getCurrentIndex()) {
                counter.increaseCount();
            }
        }
        if (element == null && shouldExist) {
            element = factory.element(name, parent.getNamespace());
            insertAtPreferredLocation(parent, element, counter);
            counter.increaseCount();
        }
        if (!shouldExist && element != null) {
            parent.removeChild(name, parent.getNamespace());
        }
        return element;
    } //-- Element updateElement(Counter, Element, String, boolean) 

    /**
     * Method updateIdentifiableBase
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateIdentifiableBase(IdentifiableBase value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
        }
    } //-- void updateIdentifiableBase(IdentifiableBase, String, Counter, Element) 

    /**
     * Method updateMirror
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateMirror(Mirror value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "mirrorOf", value.getMirrorOf());
        findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
        findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
        findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
    } //-- void updateMirror(Mirror, String, Counter, Element) 

    /**
     * Method updateProfile
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateProfile(Profile value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        updateActivation( value.getActivation(), "activation", innerCount, root);
        findAndReplaceProperties(innerCount, root,  "properties", value.getProperties());
        iterateRepository(innerCount, root, value.getRepositories());
        iterateRepository(innerCount, root, value.getPluginRepositories());
        findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
    } //-- void updateProfile(Profile, String, Counter, Element) 

    /**
     * Method updateProxy
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateProxy(Proxy value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "active", value.isActive() == false ? null : String.valueOf( value.isActive() ));
        findAndReplaceSimpleElement(innerCount, root,  "protocol", value.getProtocol() == null || value.getProtocol().equals( "http" ) ? null : value.getProtocol());
        findAndReplaceSimpleElement(innerCount, root,  "username", value.getUsername());
        findAndReplaceSimpleElement(innerCount, root,  "password", value.getPassword());
        findAndReplaceSimpleElement(innerCount, root,  "port", value.getPort() == 8080 ? null : String.valueOf( value.getPort() ));
        findAndReplaceSimpleElement(innerCount, root,  "host", value.getHost());
        findAndReplaceSimpleElement(innerCount, root,  "nonProxyHosts", value.getNonProxyHosts());
        findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
    } //-- void updateProxy(Proxy, String, Counter, Element) 

    /**
     * Method updateRepository
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateRepository(Repository value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        updateRepositoryPolicy( value.getReleases(), "releases", innerCount, root);
        updateRepositoryPolicy( value.getSnapshots(), "snapshots", innerCount, root);
        findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
        findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
        findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
        findAndReplaceSimpleElement(innerCount, root,  "layout", value.getLayout() == null || value.getLayout().equals( "default" ) ? null : value.getLayout());
    } //-- void updateRepository(Repository, String, Counter, Element) 

    /**
     * Method updateRepositoryBase
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateRepositoryBase(RepositoryBase value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
            findAndReplaceSimpleElement(innerCount, root,  "name", value.getName());
            findAndReplaceSimpleElement(innerCount, root,  "url", value.getUrl());
            findAndReplaceSimpleElement(innerCount, root,  "layout", value.getLayout() == null || value.getLayout().equals( "default" ) ? null : value.getLayout());
        }
    } //-- void updateRepositoryBase(RepositoryBase, String, Counter, Element) 

    /**
     * Method updateRepositoryPolicy
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateRepositoryPolicy(RepositoryPolicy value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
            findAndReplaceSimpleElement(innerCount, root,  "enabled", value.isEnabled() == true ? null : String.valueOf( value.isEnabled() ));
            findAndReplaceSimpleElement(innerCount, root,  "updatePolicy", value.getUpdatePolicy());
            findAndReplaceSimpleElement(innerCount, root,  "checksumPolicy", value.getChecksumPolicy());
        }
    } //-- void updateRepositoryPolicy(RepositoryPolicy, String, Counter, Element) 

    /**
     * Method updateServer
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateServer(Server value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "username", value.getUsername());
        findAndReplaceSimpleElement(innerCount, root,  "password", value.getPassword());
        findAndReplaceSimpleElement(innerCount, root,  "privateKey", value.getPrivateKey());
        findAndReplaceSimpleElement(innerCount, root,  "passphrase", value.getPassphrase());
        findAndReplaceSimpleElement(innerCount, root,  "filePermissions", value.getFilePermissions());
        findAndReplaceSimpleElement(innerCount, root,  "directoryPermissions", value.getDirectoryPermissions());
        findAndReplaceXpp3DOM(innerCount, root, "configuration", (Xpp3Dom)value.getConfiguration());
        findAndReplaceSimpleElement(innerCount, root,  "id", value.getId());
    } //-- void updateServer(Server, String, Counter, Element) 

    /**
     * Method updateSettings
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateSettings(Settings value, String xmlTag, Counter counter, Element element)
    {
        Element root = element;
        Counter innerCount = new Counter();
        findAndReplaceSimpleElement(innerCount, root,  "localRepository", value.getLocalRepository());
        findAndReplaceSimpleElement(innerCount, root,  "interactiveMode", value.isInteractiveMode() == true ? null : String.valueOf( value.isInteractiveMode() ));
        findAndReplaceSimpleElement(innerCount, root,  "usePluginRegistry", value.isUsePluginRegistry() == false ? null : String.valueOf( value.isUsePluginRegistry() ));
        findAndReplaceSimpleElement(innerCount, root,  "offline", value.isOffline() == false ? null : String.valueOf( value.isOffline() ));
        iterateProxy(innerCount, root, value.getProxies());
        iterateServer(innerCount, root, value.getServers());
        iterateMirror(innerCount, root, value.getMirrors());
        iterateProfile(innerCount, root, value.getProfiles());
        findAndReplaceSimpleLists(innerCount, root, value.getActiveProfiles(), "activeProfiles", "activeProfile");
        findAndReplaceSimpleLists(innerCount, root, value.getPluginGroups(), "pluginGroups", "pluginGroup");
    } //-- void updateSettings(Settings, String, Counter, Element) 

    /**
     * Method updateTrackableBase
     * 
     * @param value
     * @param element
     * @param counter
     * @param xmlTag
     */
    public void updateTrackableBase(TrackableBase value, String xmlTag, Counter counter, Element element)
    {
        boolean shouldExist = value != null;
        Element root = updateElement(counter, element, xmlTag, shouldExist);
        if (shouldExist) {
            Counter innerCount = new Counter();
        }
    } //-- void updateTrackableBase(TrackableBase, String, Counter, Element) 

    /**
     * Method write
     * 
     * @param settings
     * @param stream
     * @param document
     */
    public void write(Settings settings, Document document, OutputStream stream)
        throws java.io.IOException
    {
        updateSettings(settings, "settings", new Counter(), document.getRootElement());
        XMLOutputter outputter = new XMLOutputter();
        outputter.setFormat(Format.getPrettyFormat()
        .setIndent("    ")
        .setLineSeparator(System.getProperty("line.separator")));
        outputter.output(document, stream);
    } //-- void write(Settings, Document, OutputStream) 


      //-----------------/
     //- Inner Classes -/
    //-----------------/

    /**
     * Class Counter.
     * 
     * @version $Revision$ $Date$
     */
    public class Counter {


          //--------------------------/
         //- Class/Member Variables -/
        //--------------------------/

        /**
         * Field currentIndex
         */
        private int currentIndex = 0;


          //-----------/
         //- Methods -/
        //-----------/

        /**
         * Method getCurrentIndex
         */
        public int getCurrentIndex()
        {
            return currentIndex;
        } //-- int getCurrentIndex() 

        /**
         * Method increaseCount
         */
        public void increaseCount()
        {
            currentIndex = currentIndex + 1;
        } //-- void increaseCount() 

    }

}
